@@ -1,7 +1,11 @@
 
-checkxy <- function(y, x, na.option) {
+checkxy <- function(y, x, na.option, weights = NULL) {
   if (length(y) != nrow(x))
     stop("Mismatch in length of 'y' and number of rows in 'x'", call. = FALSE)
+  if (!is.null(weights)) {
+    if (length(y) != length(weights)) stop("Mismatch in 'y' and 'weights'")
+    if (any(weights < 0)) stop("Weights must be positive")
+  }
   nay <- is.na(y)
   if (any(nay)) message("'y' contains ", sum(nay), " NA")
   naxr <- !complete.cases(x)
